@@ -193,7 +193,7 @@ sub _vimeo_list_info {
             $tmp_info->{$ex}{$video_id} = {
                 video_id    => $video_id,
                 title       => $entry->{title},
-                published   => $entry->{published},
+                upload_date => $entry->{upload_date},
                 uploader    => $entry->{uploader},
                 webpage_url => $entry->{webpage_url},
             };
