@@ -42,14 +42,12 @@ sub choose_videos {
     my $has_view_count     = 0;
     my $has_duration       = 0;
     for my $video_id ( keys %{$tmp->{$ex}} ) {
-        $has_duration++                                                if $tmp->{$ex}{$video_id}{duration} ne '-:--:--';
-        $has_date_sort++                                               if $tmp->{$ex}{$video_id}{date_sort};
-        $has_view_count++                                              if $tmp->{$ex}{$video_id}{view_count};
-        $length_video_id   = length $video_id                          if length $video_id > $length_video_id;
-        $length_view_count = length $tmp->{$ex}{$video_id}{view_count} if length $tmp->{$ex}{$video_id}{view_count} > $length_view_count;
-        if ( $tmp->{$ex}{$video_id}{upload_date} ne '0000-00-00' && length $tmp->{$ex}{$video_id}{upload_date} > $length_upload_date ) {
-            $length_upload_date = length $tmp->{$ex}{$video_id}{upload_date};
-        }
+        $has_duration++   if $tmp->{$ex}{$video_id}{duration} ne '-:--:--';
+        $has_date_sort++  if $tmp->{$ex}{$video_id}{date_sort};
+        $has_view_count++ if $tmp->{$ex}{$video_id}{view_count};
+        $length_video_id    = length $video_id                           if length $video_id > $length_video_id;
+        $length_view_count  = length $tmp->{$ex}{$video_id}{view_count}  if length $tmp->{$ex}{$video_id}{view_count} > $length_view_count;
+        $length_upload_date = length $tmp->{$ex}{$video_id}{upload_date} if length $tmp->{$ex}{$video_id}{upload_date} > $length_upload_date;
     }
     if ( $length_upload_date && $length_upload_date < 10 ) {
         $length_upload_date = 10;
