@@ -8,11 +8,13 @@ use 5.010000;
 use Exporter qw( import );
 our @EXPORT_OK = qw( download_youtube );
 
-use Fcntl          qw( LOCK_EX SEEK_END );
-use File::Basename qw( basename );
-use Time::HiRes    qw( gettimeofday tv_interval );
+use Fcntl                 qw( LOCK_EX SEEK_END ); # ###
+use File::Basename        qw( basename );
+use File::Spec::Functions qw( catfile );
+use Time::HiRes           qw( gettimeofday tv_interval );
 
 use Encode::Locale     qw();
+use List::MoreUtils    qw( any );
 use LWP::UserAgent     qw();
 use Term::ANSIScreen   qw( :cursor :screen );
 use Term::Choose::Util qw( insert_sep );
@@ -20,7 +22,8 @@ use Term::Choose::Util qw( insert_sep );
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::YTDL::GetData qw( get_new_video_url );
-use App::YTDL::Helper  qw( sec_to_time encode_fs check_mapping_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR );
+use App::YTDL::Helper  qw( sec_to_time encode_fs check_mapping_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR
+                           read_json write_json );
 
 END { print SHOW_CURSOR }
 
@@ -32,12 +35,21 @@ sub download_youtube {
     for my $ex ( sort keys %$info ) {
         for my $video_id ( sort { $info->{$ex}{$a}{count} <=> $info->{$ex}{$b}{count} } keys %{$info->{$ex}} ) {
             for my $fmt ( sort keys %{$info->{$ex}{$video_id}{file_name}} ) {
-                $nr++;
                 my $file_name = $info->{$ex}{$video_id}{file_name}{$fmt};
+                $nr++;
+                if ( $opt->{skip_archived_videos} && _is_in_download_archive( $opt, $info, $ex, $video_id, $fmt ) ) {
+                    push @{$opt->{skipped_archived_videos}}, sprintf "%s - %s", $video_id, $info->{$ex}{$video_id}{title}; #
+                    my $video_count = sprintf "%*s from %s", length $opt->{nr_videos}, $nr, $opt->{nr_videos};
+                    printf "  %s\n", '-' x length $video_count if $nr > 1;
+                    printf "  %s (%s)\n", check_mapping_stdout( $opt, basename $file_name ), $info->{$ex}{$video_id}{duration} // '?';
+                    printf "  %s   Skipped\n", $video_count;
+                    next;
+                }
                 if ( ! eval {
-                    unlink encode_fs( $opt, $file_name ) or die encode_fs( $opt, $file_name ) . " $!" if -f encode_fs( $opt, $file_name ) && $opt->{overwrite};
-                    _download_video( $opt, $info, $ex, $video_id, $fmt, $nr );
-                    timestamp_to_upload_date( $opt, $info, $ex, $video_id, $file_name ) if $opt->{modify_timestamp};
+                    unlink encode_fs( $opt, $file_name ) or die encode_fs( $opt, $file_name ) . " $!" if $opt->{overwrite} && -f encode_fs( $opt, $file_name );
+                    my $ok = _download_video( $opt, $info, $ex, $video_id, $fmt, $nr );
+                    timestamp_to_upload_date( $opt, $info, $ex, $video_id, $file_name )               if $opt->{modify_timestamp};
+                    _archive_download(  $opt, $info, $ex, $video_id, $fmt )                           if $opt->{enable_download_archive} && $ok && $ok == 1;
                     1 }
                 ) {
                     say "$video_id - ", check_mapping_stdout( $opt, $@ );
@@ -51,8 +63,7 @@ sub download_youtube {
 
 sub _download_video {
     my ( $opt, $info, $ex, $video_id, $fmt, $nr ) = @_;
-    my $file_name     = $info->{$ex}{$video_id}{file_name}{$fmt};
-    my $file_basename = basename $file_name;
+    my $file_name = $info->{$ex}{$video_id}{file_name}{$fmt};
     my $ua = LWP::UserAgent->new(
         agent         => $opt->{useragent},
         timeout       => $opt->{timeout},
@@ -61,12 +72,13 @@ sub _download_video {
     print HIDE_CURSOR;
     my $video_count = sprintf "%*s from %s", length $opt->{nr_videos}, $nr, $opt->{nr_videos};
     printf "  %s\n", '-' x length $video_count if $nr > 1;
-    printf "  %s (%s)\n", check_mapping_stdout( $opt, $file_basename ), $info->{$ex}{$video_id}{duration} // '?';
+    printf "  %s (%s)\n", check_mapping_stdout( $opt, basename $file_name ), $info->{$ex}{$video_id}{duration} // '?';
     local $SIG{INT} = sub {
         print cldown, "\n";
         print SHOW_CURSOR;
         exit( 1 );
     };
+    my $ok; #
     my $p = {};
     my $try = 1;
 
@@ -104,7 +116,7 @@ sub _download_video {
             );
             close $fh or die $!;
         }
-        my $ok = _status( $opt, $info, $ex, $video_id, $fmt, $p, $res, $video_count, $try, $retries );
+        $ok = _status( $opt, $info, $ex, $video_id, $fmt, $p, $res, $video_count, $try, $retries );
         if ( $ok ) {
             last RETRY;
         }
@@ -116,7 +128,7 @@ sub _download_video {
         sleep 4 * $try;
     }
     print SHOW_CURSOR;
-    return;
+    return $ok;
 }
 
 
@@ -152,7 +164,7 @@ sub _status {
             $size      = ' ' x length $size;
             $avg_speed = ' ' x length $avg_speed;
             printf _p_fmt( $opt, "status_416" ), $video_count, $retries, $dl_time, $at, $pr_status;
-            return 1;
+            return 2;
         }
         else {
             if ( $status == 200 ) {
@@ -211,23 +223,23 @@ sub _p_fmt {
 sub _return_callback {
     my ( $opt, $fh, $p ) = @_;
     my $time = $p->{starttime};
-    my ( $inter, $kbs, $chunk_size, $download, $eta ) = ( 0 ) x 5;
+    my ( $interval, $kbs, $chunk_size, $download, $eta ) = ( 0 ) x 5;
     my $resume_size = $p->{size} // 0;
     return sub {
         my ( $chunk, $res, $proto ) = @_;
-        $inter += tv_interval( [ $time ] );
+        $interval += tv_interval( [ $time ] );
         print $fh $chunk;
         my $received = tell $fh;
         $chunk_size += length $chunk;
         $download = $received - $resume_size;
         $p->{total} = $res->header( 'Content-Length' ) // 0;
-        if ( $download > 0 && $inter > 2 ) {
+        if ( $download > 0 && $interval > 2 ) {
             $p->{kbs_avg} = ( $download / 1024 ) / tv_interval[ $p->{starttime} ];
             $eta = sec_to_time( int( ( $p->{total} - $download ) / ( $p->{kbs_avg} * 1024 ) ), 1 );
             $p->{kbs_avg} = int( $p->{kbs_avg} );
             $eta = undef if ! $p->{kbs_avg};
-            $kbs = int( ( $chunk_size / 1024 ) / $inter );
-            $inter = 0;
+            $kbs = int( ( $chunk_size / 1024 ) / $interval );
+            $interval = 0;
             $chunk_size = 0;
         }
         my ( $info1, $info2 );
@@ -261,7 +273,7 @@ sub _return_callback {
 }
 
 
-sub _log_info {
+sub _log_info { # ###
     my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
     my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
     my $uploader_id = $info->{$ex}{$video_id}{uploader_id};
@@ -281,6 +293,39 @@ sub _log_info {
 }
 
 
+sub _archive_download {
+    my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
+    my $filename = catfile $opt->{archive_dir}, sprintf 'downloads_%s.json', $ex;
+    my $archive = read_json( $opt, $filename ) // [];
+    if ( $opt->{enable_download_archive} == 2 ) {
+        my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
+        push @$archive, {
+            video_id    => $video_id,
+            title       => $info->{$ex}{$video_id}{title},
+            datetime    => sprintf( "%04d-%02d-%02dT%02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min ),
+            upload_date => $info->{$ex}{$video_id}{upload_date},
+            uploader    => sprintf( "%s (%s)", $info->{$ex}{$video_id}{uploader}, $info->{$ex}{$video_id}{uploader_id} ),
+            fmt         => $fmt,
+        };
+    }
+    else {
+        push @$archive, { video_id => $video_id};
+    }
+    write_json( $opt, $filename, $archive );
+}
+
+
+sub _is_in_download_archive{
+    my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
+    my $filename = catfile $opt->{archive_dir}, sprintf 'downloads_%s.json', $ex;
+    return   if ! -f encode_fs( $opt, $filename );
+    my $archive = read_json( $opt, $filename );
+    return 1 if any { ( $_->{video_id} // '' ) eq $video_id } @$archive;
+    return;
+}
+
+
+
 
 
 1;
