@@ -10,6 +10,7 @@ our @EXPORT_OK = qw( sec_to_time get_filename timestamp_to_upload_date encode_fs
                     write_json read_json sanitize_for_path uni_capture uni_system HIDE_CURSOR SHOW_CURSOR );
 
 use Encode             qw( encode FB_CROAK LEAVE_SRC );
+use Fcntl              qw( LOCK_EX LOCK_SH SEEK_END );
 use Time::Local        qw( timelocal );
 use Unicode::Normalize qw( NFC );
 
@@ -168,6 +169,8 @@ sub write_json {
     my ( $opt, $file, $h_ref ) = @_;
     my $json = JSON->new->pretty->canonical->utf8->encode( $h_ref );
     open my $fh, '>', encode_fs( $opt, $file ) or die encode_fs( $opt, $file ) . " $!";
+    flock $fh, LOCK_EX     or die $!;
+    seek  $fh, 0, SEEK_END or die $!;
     print $fh $json;
     close $fh;
 }
@@ -175,8 +178,9 @@ sub write_json {
 
 sub read_json {
     my ( $opt, $file ) = @_;
-    return {} if ! -f encode_fs( $opt, $file );
+    return if ! -f encode_fs( $opt, $file );
     open my $fh, '<', encode_fs( $opt, $file ) or die encode_fs( $opt, $file ) . " $!";
+    flock $fh, LOCK_SH or die $!;
     my $json = do { local $/; <$fh> };
     close $fh;
     my $h_ref = JSON->new->utf8->decode( $json ) if $json;
@@ -185,6 +189,8 @@ sub read_json {
 
 
 
+
+
 1;
 
 
