@@ -7,6 +7,9 @@ use 5.010000;
 
 use parent qw( LWP::UserAgent );
 
+use App::YTDL::Helper qw( HIDE_CURSOR SHOW_CURSOR );
+
+
 
 
 my @ANI = qw(- \ | /);
@@ -14,6 +17,7 @@ my @ANI = qw(- \ | /);
 sub progress {
     my( $self, $status, $m ) = @_;
     return unless $self->{show_progress};
+    print STDERR HIDE_CURSOR;
     local( $,, $\ );
     if ( $status eq "begin" ) {
         my $uri = $m->uri;
@@ -48,6 +52,7 @@ sub progress {
         $self->{progress_lastp} = $p;
     }
     STDERR->flush;
+    print STDERR SHOW_CURSOR;
 }
 
 
