@@ -49,23 +49,25 @@ sub _show_info {
             $ffprobe_version = '';
         }
     };
-    my $version     = '  version ';
-    my $bin         = '    bin   ';
-    my $video_dir   = ' video dir';
-    my $config_dir  = 'config dir';
-    my $youtube_dl  = 'youtube-dl';
-    my $ffmpeg      = '  ffmpeg  ';
-    my $ffprobe     = '  ffprobe ';
+    my $version     = '  version  ';
+    my $bin         = '    bin    ';
+    my $video_dir   = ' video dir ';
+    my $config_dir  = 'config dir ';
+    my $archive_dir = 'archive dir';
+    my $youtube_dl  = 'youtube-dl ';
+    my $ffmpeg      = '  ffmpeg   ';
+    my $ffprobe     = '  ffprobe  ';
     my $path = {
-        $version    => $main::VERSION,
-        $bin        => catfile( $RealBin, $RealScript ),
-        $video_dir  => $opt->{video_dir},
-        $config_dir => $opt->{config_dir},
-        $youtube_dl => $youtube_dl_version // 'error',
-        $ffmpeg     => $ffmpeg_version,
-        $ffprobe    => $ffprobe_version,
+        $version     => $main::VERSION,
+        $bin         => catfile( $RealBin, $RealScript ),
+        $video_dir   => $opt->{video_dir},
+        $config_dir  => $opt->{config_dir},
+        $archive_dir => $opt->{archive_dir},
+        $youtube_dl  => $youtube_dl_version // 'error',
+        $ffmpeg      => $ffmpeg_version,
+        $ffprobe     => $ffprobe_version,
     };
-    my $keys = [ $bin, $version, $video_dir, $config_dir, $youtube_dl ];
+    my $keys = [ $bin, $version, $video_dir, $config_dir, $archive_dir, $youtube_dl ];
     push @$keys, $ffmpeg  if $ffmpeg_version;
     push @$keys, $ffprobe if $ffprobe_version;
     print_hash( $path, { keys => $keys, preface => ' Close with ENTER' } );
@@ -105,10 +107,11 @@ sub _menus {
             [ '_print_pref_qual', "  Preferred qualities"          ],
         ],
         group_download => [
-            [ 'useragent',      "- UserAgent"        ],
-            [ 'overwrite',      "- Overwrite"        ],
-            [ 'retries',        "- Download retries" ],
-            [ 'timeout',        "- Timeout"          ],
+            [ 'useragent',            "- UserAgent"            ],
+            [ 'skip_archived_videos', "- Skip archived videos" ],
+            [ 'overwrite',            "- Overwrite"            ],
+            [ 'retries',              "- Download retries"     ],
+            [ 'timeout',              "- Timeout"              ],
         ],
         group_merge => [
             [ 'merge_enabled',   "- Enable Merge"      ],
@@ -118,9 +121,10 @@ sub _menus {
             [ 'merged_in_files', "- Input files"       ],
         ],
         group_history => [
-            [ 'log_info',                  "- Logging"         ],
-            [ 'max_size_history',          "- Size history" ],
-            [ 'sort_history_by_timestamp', "- History sort"    ],
+            [ 'log_info',                  "- Logging"          ], # ### pod
+            [ 'max_size_history',          "- Size history"     ],
+            [ 'sort_history_by_timestamp', "- History sort"     ],
+            [ 'enable_download_archive',   "- Download archive" ],
         ],
 
         group_list_menu => [
@@ -316,7 +320,7 @@ sub set_options {
                 _opt_choose_from_list_value( $opt, $choice, $prompt, $list );
             }
             elsif ( $choice eq "_print_pref_qual" ) {
-                my $pref = read_json( $opt, $opt->{preferred_file} );
+                my $pref = read_json( $opt, $opt->{preferred_file} ) // {};
                 if ( ! %$pref ) {
                     choose( [ 'Close with ENTER' ], { prompt => 'No preferred qualities set.' } );
                 }
@@ -345,6 +349,11 @@ sub set_options {
                 $opt->{useragent} = 'Mozilla/5.0' if $opt->{useragent} eq '';
                 $opt->{useragent} = ''            if $opt->{useragent} eq '""';
             }
+            elsif ( $choice eq "skip_archived_videos" ) {
+                my $prompt = 'Skip archived videos';
+                my $list = [ qw( no yes ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
             elsif ( $choice eq "overwrite" ) {
                 my $prompt = 'Overwrite files';
                 my $list = [
@@ -392,7 +401,7 @@ sub set_options {
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
-            elsif ( $choice eq "log_info" ) {
+            elsif ( $choice eq "log_info" ) { # ###
                 my $prompt = 'Enable info-logging';
                 my $list = [ qw( no yes ) ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
@@ -410,6 +419,11 @@ sub set_options {
                 ];
                 _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
             }
+            elsif ( $choice eq "enable_download_archive" ) {
+                my $prompt = 'Enable downlaod archive';
+                my $list = [ qw( no yes more ) ];
+                _opt_choose_from_list_idx( $opt, $choice, $prompt, $list );
+            }
             elsif ( $choice eq "_fast_listmenu" ) {
                 my $sub_menu = _sub_menus( $choice );
                 my $current = { map { $_->[0] => $opt->{$_->[0]} } @$sub_menu };
@@ -594,7 +608,7 @@ sub _write_config_file {
 
 sub read_config_file {
     my ( $opt, $file ) = @_;
-    my $tmp = read_json( $opt, $file );
+    my $tmp = read_json( $opt, $file ) // {};
     my @keys = keys %$tmp;
     for my $section ( @keys ) {
         $opt->{$section} = $tmp->{$section};
